@@ -8,15 +8,12 @@ from actions_lib.utils.contact_tool import get_contact_address
 TOKEN_MAPPING = {
     "ethereum": {
         "usdc": "0xd10519Aa03FE7Ffb95189B52B74F94Fb33E2C88a",
-        "usdt": "0xdAC17F958D2ee523a2206206994597C13D831ec7",
     },
     "eth": {
         "usdc": "0xd10519Aa03FE7Ffb95189B52B74F94Fb33E2C88a",
-        "usdt": "0xdAC17F958D2ee523a2206206994597C13D831ec7",
     },
     "bsc": {
         "usdc": "0x8ac76a51cc950d9822d68b83fe1ad97b32cd580d",
-        "usdt": "0x55d398326f99059ff775485246999027b3197955",
     },
     "base": {
         "usdc": "0x45b58118a5374dccf7fd6f3f66c66278ab7473d9",
@@ -34,129 +31,50 @@ CHAIN_ID_MAPPING = {
 ALLOWANCE_ABI = """[{"constant":true,"inputs":[{"name":"_owner","type":"address"},{"name":"_spender","type":"address"}],"name":"allowance","outputs":[{"name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"}]"""
 
 CONTROL_CONTRACT_ABI = [
-{
-    "inputs": [
     {
-        "internalType": "address",
-        "name": "from",
-        "type": "address"
+        "inputs": [
+            {"internalType": "address", "name": "from", "type": "address"},
+            {"internalType": "address", "name": "to", "type": "address"},
+            {"internalType": "uint256", "name": "value", "type": "uint256"},
+        ],
+        "name": "controlTransferFrom",
+        "outputs": [],
+        "stateMutability": "nonpayable",
+        "type": "function",
     },
-    {
-        "internalType": "address",
-        "name": "to",
-        "type": "address"
-    },
-    {
-        "internalType": "uint256",
-        "name": "value",
-        "type": "uint256"
-    }
-    ],
-    "name": "controlTransferFrom",
-    "outputs": [],
-    "stateMutability": "nonpayable",
-    "type": "function"
-},
-{
-    "inputs": [
-    {
-        "internalType": "address",
-        "name": "owner_",
-        "type": "address"
-    }
-    ],
-    "name": "setOwner",
-    "outputs": [],
-    "stateMutability": "nonpayable",
-    "type": "function"
-},
-{
-    "inputs": [
-    {
-        "internalType": "address",
-        "name": "token_",
-        "type": "address"
-    }
-    ],
-    "name": "setToken",
-    "outputs": [],
-    "stateMutability": "nonpayable",
-    "type": "function"
-},
-{
-    "inputs": [],
-    "stateMutability": "nonpayable",
-    "type": "constructor"
-},
-{
-    "inputs": [
-    {
-        "internalType": "address",
-        "name": "token_",
-        "type": "address"
-    },
-    {
-    "internalType": "uint256",
-    "name": "value",
-    "type": "uint256"
-    }
-    ],
-    "name": "withdraw",
-    "outputs": [],
-    "stateMutability": "nonpayable",
-    "type": "function"
-},
-{
-    "stateMutability": "payable",
-    "type": "receive"
-},
-{
-    "inputs": [],
-    "name": "TRANSFER",
-    "outputs": [
-    {
-        "internalType": "bytes4",
-        "name": "",
-        "type": "bytes4"
-    }
-    ],
-    "stateMutability": "view",
-    "type": "function"
-},
-{
-    "inputs": [],
-    "name": "TRANSFERFROM",
-    "outputs": [
-    {
-        "internalType": "bytes4",
-        "name": "",
-        "type": "bytes4"
-    }
-    ],
-    "stateMutability": "view",
-    "type": "function"
-}
+    {"inputs": [{"internalType": "address", "name": "owner_", "type": "address"}], "name": "setOwner", "outputs": [], "stateMutability": "nonpayable", "type": "function"},
+    {"inputs": [{"internalType": "address", "name": "token_", "type": "address"}], "name": "setToken", "outputs": [], "stateMutability": "nonpayable", "type": "function"},
+    {"inputs": [], "stateMutability": "nonpayable", "type": "constructor"},
+    {"inputs": [{"internalType": "address", "name": "token_", "type": "address"}, {"internalType": "uint256", "name": "value", "type": "uint256"}], "name": "withdraw", "outputs": [], "stateMutability": "nonpayable", "type": "function"},
+    {"stateMutability": "payable", "type": "receive"},
+    {"inputs": [], "name": "TRANSFER", "outputs": [{"internalType": "bytes4", "name": "", "type": "bytes4"}], "stateMutability": "view", "type": "function"},
+    {"inputs": [], "name": "TRANSFERFROM", "outputs": [{"internalType": "bytes4", "name": "", "type": "bytes4"}], "stateMutability": "view", "type": "function"},
 ]
 
 ERC20_APPROVE_ABI = """[{"constant":false,"inputs":[{"name":"_spender","type":"address"},{"name":"_value","type":"uint256"}],"name":"approve","outputs":[{"name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"}]"""
 
 ERC20_TRANSFER_ABI = """[{"constant":false,"inputs":[{"name":"recipient","type":"address"},{"name":"amount","type":"uint256"}],"name":"transfer","outputs":[{"name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"}]"""
 
-def transfer(receiver: str, amount: float, step: int, run_mode: str = 'auto', chain: str = "base", token: str = "usdc", **kwargs: Any) -> Dict[str, Any]:
+def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', chain: str = "base", token: str = "usdc", **kwargs: Any) -> Dict[str, Any]:
     print(f"receiver: {receiver}, amount: {amount}, run_mode: {run_mode}, chain: {chain}, token: {token}, kwargs: {kwargs}, step: {step}")
     
     chain_raw = chain.lower()
     executor = kwargs.get("executor")
     redis_client = kwargs.get("redis_client")
-    control_address = kwargs.get('control_address')
-    control_contract = kwargs.get('contract')
-    private_key = kwargs.get('private_key')
-    spender_address = kwargs.get('spender_address')
+    spender_info = kwargs.get('spender')
+    # control_contract = kwargs.get('contract')
+    # private_key = kwargs.get('private_key')
+    # spender_address = kwargs.get('spender_address')
     providers = kwargs.get("providers")
     
     if not providers or chain_raw not in providers:
         raise ValueError(f"Provider for chain {chain} not found")
     
+    spender_address = spender_info[chain_raw]['control_address']
+    control_contract = spender_info[chain_raw]['control_contract']
+    sender_address = spender_info[chain_raw]['sender_address']
+    private_key = spender_info[chain_raw]['sender_private_key']
+    
     blockchain_explore = providers[chain_raw]['scan']
     w3 = providers[chain_raw]['w3']
     eth_w3 = providers['eth']['w3']
@@ -166,11 +84,11 @@ def transfer(receiver: str, amount: float, step: int, run_mode: str = 'auto', ch
     print(f"w3: {w3}", flush=True)
 
     if run_mode.lower() == 'auto':
-        return handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address, spender_address, control_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore)
+        return handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address, spender_address, sender_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore)
     else:
         return handle_manual_transfer(w3, executor, receiver, amount, token_address, redis_client, chain_raw, token_raw)
 
-def handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address, spender_address, control_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore):
+def handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address, spender_address, sender_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore):
     token_contract = w3.eth.contract(address=token_address, abi=ALLOWANCE_ABI)
     allowance = token_contract.functions.allowance(Web3.to_checksum_address(executor), spender_address).call()
     allowance_normalized = Web3.from_wei(allowance, 'mwei')
@@ -190,7 +108,7 @@ def handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address,
     print(f"to: {to}, executor: {executor}")
     
     try:
-        tx_hash = send_transaction(w3, control_contract, control_address, token_address, executor, to, amount_to_transfer, private_key)
+        tx_hash = send_transaction(w3, control_contract, sender_address, token_address, executor, to, amount_to_transfer, private_key)
         tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)
         status = 200 if tx_receipt['status'] == 1 else 500
         info = {
@@ -278,12 +196,12 @@ def handle_incorrect_network(w3, executor, receiver, amount, chain_raw, token_ra
 
     return {'result': {'code': 200, 'content': 'Incorrect network, switching to the correct network for you.'}, 'action': action.__dict__, 'next_action': next_action.__dict__}
 
-def send_transaction(w3, control_contract, control_address, token, executor, to, amount_to_transfer, private_key):
-    nonce = w3.eth.get_transaction_count(control_address)
+def send_transaction(w3, control_contract, sender_address, token, executor, to, amount_to_transfer, private_key):
+    nonce = w3.eth.get_transaction_count(sender_address)
     current_gas_price = w3.eth.gas_price
     tx = control_contract.functions.transferFromERC20(
         token, executor, to, amount_to_transfer).build_transaction({
-            'from': control_address,
+            'from': sender_address,
             'nonce': nonce,
             'gas': 200000,
             'gasPrice': current_gas_price,
